@@ -14,14 +14,16 @@ def configuration(parent_package='',top_path=None):
     config.add_library('dfftpack',
                        sources=[join('src/dfftpack','*.f')])
 
+    config.add_library('fftpack',
+                       sources=[join('src/fftpack','*.f')])
+
     sources = ['fftpack.pyf','src/zfft.c','src/drfft.c','src/zrfft.c',
-               'src/zfftnd.c']
+               'src/zfftnd.c', 'src/dct.c.src', 'src/dst.c.src']
 
     config.add_extension('_fftpack',
         sources=sources,
-        libraries=['dfftpack'],
-        depends=['src/zfft_fftpack.c', 'src/drfft_fftpack.c',
-                 'src/zfftnd_fftpack.c'])
+        libraries=['dfftpack', 'fftpack'],
+        include_dirs=['src'])
 
     config.add_extension('convolve',
         sources=['convolve.pyf','src/convolve.c'],
