@@ -6,7 +6,7 @@
 __all__ = ['interp1d', 'interp2d', 'spline', 'spleval', 'splmake', 'spltopp',
            'ppform', 'lagrange']
 
-from numpy import shape, sometrue, rank, array, transpose, searchsorted, \
+from numpy import shape, sometrue, array, transpose, searchsorted, \
                   ones, logical_or, atleast_1d, atleast_2d, meshgrid, ravel, \
                   dot, poly1d, asarray, intp
 import numpy as np
@@ -23,10 +23,22 @@ def reduce_sometrue(a):
     return all
 
 def lagrange(x, w):
-    """Return the Lagrange interpolating polynomial of the data-points (x,w)
+    """
+    Return a Lagrange interpolating polynomial.
 
-    Warning: This implementation is numerically unstable; do not expect to
+    Given two 1-D arrays `x` and `w,` returns the Lagrange interpolating
+    polynomial through the points ``(x, w)``.
+
+    Warning: This implementation is numerically unstable. Do not expect to
     be able to use more than about 20 points even if they are chosen optimally.
+
+    Parameters
+    ----------
+    x : array_like
+        `x` represents the x-coordinates of a set of datapoints.
+    w : array_like
+        `w` represents the y-coordinates of a set of datapoints, i.e. f(`x`).
+
     """
     M = len(x)
     p = poly1d(0.0)
@@ -48,46 +60,83 @@ class interp2d(object):
     interp2d(x, y, z, kind='linear', copy=True, bounds_error=False,
              fill_value=nan)
 
-    Interpolate over a 2D grid.
+    Interpolate over a 2-D grid.
+
+    `x`, `y` and `z` are arrays of values used to approximate some function
+    f: ``z = f(x, y)``. This class returns a function whose call method uses
+    spline interpolation to find the value of new points.
+
+    Methods
+    -------
+    __call__
 
     Parameters
     ----------
-    x, y : 1D arrays
-        Arrays defining the coordinates of a 2D grid.  If the
-        points lie on a regular grid, `x` can specify the column coordinates
-        and `y` the row coordinates, e.g.::
+    x, y : 1-D ndarrays
+        Arrays defining the data point coordinates.
 
-            x = [0,1,2];  y = [0,3,7]
+        If the points lie on a regular grid, `x` can specify the column
+        coordinates and `y` the row coordinates, for example::
 
-        otherwise x and y must specify the full coordinates, i.e.::
+          >>> x = [0,1,2];  y = [0,3]; z = [[1,2,3], [4,5,6]]
 
-            x = [0,1,2,0,1,2,0,1,2];  y = [0,0,0,3,3,3,7,7,7]
+        Otherwise, x and y must specify the full coordinates for each point,
+        for example::
+
+          >>> x = [0,1,2,0,1,2];  y = [0,0,0,3,3,3]; z = [1,2,3,4,5,6]
 
         If `x` and `y` are multi-dimensional, they are flattened before use.
 
-    z : 1D array
-        The values of the interpolated function on the grid points. If
-        z is a multi-dimensional array, it is flattened before use.
-    kind : {'linear', 'cubic', 'quintic'}
-        The kind of interpolation to use.
-    copy : bool
+    z : 1-D ndarray
+        The values of the function to interpolate at the data points. If
+        `z` is a multi-dimensional array, it is flattened before use.
+    kind : {'linear', 'cubic', 'quintic'}, optional
+        The kind of spline interpolation to use. Default is 'linear'.
+    copy : bool, optional
         If True, then data is copied, otherwise only a reference is held.
-    bounds_error : bool
+    bounds_error : bool, optional
         If True, when interpolated values are requested outside of the
         domain of the input data, an error is raised.
         If False, then `fill_value` is used.
-    fill_value : number
+    fill_value : number, optional
         If provided, the value to use for points outside of the
         interpolation domain. Defaults to NaN.
 
-    Raises
-    ------
-    ValueError when inputs are invalid.
-
     See Also
     --------
-    bisplrep, bisplev : spline interpolation based on FITPACK
+    bisplrep, bisplev
+        Spline interpolation based on FITPACK
     BivariateSpline : a more recent wrapper of the FITPACK routines
+    interp1d
+
+    Notes
+    -----
+    The minimum number of data points required along the interpolation
+    axis is ``(k+1)**2``, with k=1 for linear, k=3 for cubic and k=5 for
+    quintic interpolation.
+
+    The interpolator is constructed by `bisplrep`, with a smoothing factor
+    of 0. If more control over smoothing is needed, `bisplrep` should be
+    used directly.
+
+    Examples
+    --------
+    Construct a 2-D grid and interpolate on it:
+
+    >>> from scipy import interpolate
+    >>> x = np.arange(-5.01, 5.01, 0.25)
+    >>> y = np.arange(-5.01, 5.01, 0.25)
+    >>> xx, yy = np.meshgrid(x, y)
+    >>> z = np.sin(xx**2+yy**2)
+    >>> f = interpolate.interp2d(x, y, z, kind='cubic')
+
+    Now use the obtained interpolation function and plot the result:
+
+    >>> xnew = np.arange(-5.01, 5.01, 1e-2)
+    >>> ynew = np.arange(-5.01, 5.01, 1e-2)
+    >>> znew = f(xnew, ynew)
+    >>> plt.plot(x, z[:, 0], 'ro-', xnew, znew[:, 0], 'b-')
+    >>> plt.show()
 
     """
 
@@ -144,55 +193,68 @@ class interp2d(object):
 
 
 class interp1d(object):
-    """ Interpolate a 1D function.
+    """
+    interp1d(x, y, kind='linear', axis=-1, copy=True, bounds_error=True,
+             fill_value=np.nan)
+
+    Interpolate a 1-D function.
+
+    `x` and `y` are arrays of values used to approximate some function f:
+    ``y = f(x)``.  This class returns a function whose call method uses
+    interpolation to find the value of new points.
+
+    Parameters
+    ----------
+    x : array_like
+        A 1-D array of monotonically increasing real values.
+    y : array_like
+        A N-D array of real values. The length of `y` along the interpolation
+        axis must be equal to the length of `x`.
+    kind : str or int, optional
+        Specifies the kind of interpolation as a string
+        ('linear','nearest', 'zero', 'slinear', 'quadratic, 'cubic')
+        or as an integer specifying the order of the spline interpolator
+        to use. Default is 'linear'.
+    axis : int, optional
+        Specifies the axis of `y` along which to interpolate.
+        Interpolation defaults to the last axis of `y`.
+    copy : bool, optional
+        If True, the class makes internal copies of x and y.
+        If False, references to `x` and `y` are used. The default is to copy.
+    bounds_error : bool, optional
+        If True, an error is thrown any time interpolation is attempted on
+        a value outside of the range of x (where extrapolation is
+        necessary). If False, out of bounds values are assigned `fill_value`.
+        By default, an error is raised.
+    fill_value : float, optional
+        If provided, then this value will be used to fill in for requested
+        points outside of the data range. If not provided, then the default
+        is NaN.
 
     See Also
     --------
-    splrep, splev - spline interpolation based on FITPACK
-    UnivariateSpline - a more recent wrapper of the FITPACK routines
+    UnivariateSpline : A more recent wrapper of the FITPACK routines.
+    splrep, splev
+        Spline interpolation based on FITPACK.
+    interp2d
+
+    Examples
+    --------
+    >>> from scipy import interpolate
+    >>> x = np.arange(0, 10)
+    >>> y = np.exp(-x/3.0)
+    >>> f = interpolate.interp1d(x, y)
+
+    >>> xnew = np.arange(0,9, 0.1)
+    >>> ynew = f(xnew)   # use interpolation function returned by `interp1d`
+    >>> plt.plot(x, y, 'o', xnew, ynew, '-')
+    >>> plt.show()
+
     """
 
     def __init__(self, x, y, kind='linear', axis=-1,
                  copy=True, bounds_error=True, fill_value=np.nan):
-        """ Initialize a 1D linear interpolation class.
-
-        Description
-        -----------
-        x and y are arrays of values used to approximate some function f:
-            y = f(x)
-        This class returns a function whose call method uses linear
-        interpolation to find the value of new points.
-
-        Parameters
-        ----------
-        x : array
-            A 1D array of monotonically increasing real values.  x cannot
-            include duplicate values (otherwise f is overspecified)
-        y : array
-            An N-D array of real values.  y's length along the interpolation
-            axis must be equal to the length of x.
-        kind : str or int
-            Specifies the kind of interpolation as a string ('linear',
-            'nearest', 'zero', 'slinear', 'quadratic, 'cubic') or as an integer
-            specifying the order of the spline interpolator to use.
-        axis : int
-            Specifies the axis of y along which to interpolate. Interpolation
-            defaults to the last axis of y.
-        copy : bool
-            If True, the class makes internal copies of x and y.
-            If False, references to x and y are used.
-            The default is to copy.
-        bounds_error : bool
-            If True, an error is thrown any time interpolation is attempted on
-            a value outside of the range of x (where extrapolation is
-            necessary).
-            If False, out of bounds values are assigned fill_value.
-            By default, an error is raised.
-        fill_value : float
-            If provided, then this value will be used to fill in for requested
-            points outside of the data range.
-            If not provided, then the default is NaN.
-        """
+        """ Initialize a 1D linear interpolation class."""
 
         self.copy = copy
         self.bounds_error = bounds_error
@@ -481,7 +543,8 @@ def _find_smoothest(xk, yk, order, conds=None, B=None):
     return _dot0(tmp, yk)
 
 def _setdiag(a, k, v):
-    assert (a.ndim==2)
+    if not a.ndim == 2:
+        raise ValueError("Input array should be 2-D.")
     M,N = a.shape
     if k > 0:
         start = k
@@ -654,7 +717,7 @@ def _get_spline3_Bb(xk, yk, kind, conds):
         b *= 6.0
         return B, b, None, None
     else:
-        raise ValueError, "%s not supported" % kind
+        raise ValueError("%s not supported" % kind)
 
 # conds is a tuple of an array and a vector
 #  giving the left-hand and the right-hand side
@@ -662,8 +725,8 @@ def _get_spline3_Bb(xk, yk, kind, conds):
 def _find_user(xk, yk, order, conds, B):
     lh = conds[0]
     rh = conds[1]
-    B = concatenate((B,lh),axis=0)
-    w = concatenate((yk,rh),axis=0)
+    B = np.concatenate((B,lh),axis=0)
+    w = np.concatenate((yk,rh),axis=0)
     M,N = B.shape
     if (M>N):
         raise ValueError("over-specification of conditions")
