@@ -28,6 +28,7 @@ from numpy.testing import \
      assert_array_almost_equal, \
      assert_equal, \
      assert_raises, run_module_suite
+from numpy.testing.utils import WarningManager
 
 from nose.tools import assert_true
 
@@ -197,9 +198,9 @@ case_table5.append(
     {'name': 'object',
      'expected': {'testobject': MO}
      })
-u_str = open(
-    pjoin(test_data_path, 'japanese_utf8.txt'),
-    'rb').read().decode('utf-8')
+fp_u_str = open(pjoin(test_data_path, 'japanese_utf8.txt'), 'rb')
+u_str = fp_u_str.read().decode('utf-8')
+fp_u_str.close()
 case_table5.append(
     {'name': 'unicode',
     'expected': {'testunicode': array([u_str])}
@@ -383,22 +384,28 @@ def test_mat73():
         pjoin(test_data_path, 'testhdf5*.mat'))
     assert_true(len(filenames)>0)
     for filename in filenames:
+        fp = open(filename, 'rb')
         assert_raises(NotImplementedError,
                       loadmat,
-                      filename,
+                      fp,
                       struct_as_record=True)
+        fp.close()
 
 
 def test_warnings():
     fname = pjoin(test_data_path, 'testdouble_7.1_GLNX86.mat')
-    warnings.simplefilter('error')
-    # This should not generate a warning
-    mres = loadmat(fname, struct_as_record=True)
-    # This neither
-    mres = loadmat(fname, struct_as_record=False)
-    # This should - because of deprecated system path search
-    yield assert_raises, DeprecationWarning, find_mat_file, fname
-    warnings.resetwarnings()
+    warn_ctx = WarningManager()
+    warn_ctx.__enter__()
+    try:
+        warnings.simplefilter('error')
+        # This should not generate a warning
+        mres = loadmat(fname, struct_as_record=True)
+        # This neither
+        mres = loadmat(fname, struct_as_record=False)
+        # This should - because of deprecated system path search
+        assert_raises(DeprecationWarning, find_mat_file, fname)
+    finally:
+        warn_ctx.__exit__()
 
 
 def test_regression_653():
@@ -509,43 +516,59 @@ def test_use_small_element():
 
 def test_save_dict():
     # Test that dict can be saved (as recarray), loaded as matstruct
-    d = {'a':1, 'b':2}
-    stream = BytesIO()
-    savemat_future(stream, {'dict':d})
-    stream.seek(0)
-    vals = loadmat(stream)
+    dict_types = (dict,)
+    try:
+        from collections import OrderedDict
+    except ImportError:
+        pass
+    else:
+        dict_types += (OrderedDict,)
+    exp_arr = np.zeros((1, 1), dtype = [('a', object), ('b', object)])
+    exp_arr['a'] = 1
+    exp_arr['b'] = 2
+    for dict_type in dict_types:
+        d = dict_type(a=1, b=2)
+        stream = BytesIO()
+        savemat_future(stream, {'dict': d})
+        stream.seek(0)
+        vals = loadmat(stream)
+        assert_array_equal(vals['dict'], exp_arr)
 
 
 def test_1d_shape():
     # Current 5 behavior is 1D -> column vector
     arr = np.arange(5)
     stream = BytesIO()
-    # silence warnings for tests
-    warnings.simplefilter('ignore')
-    savemat(stream, {'oned':arr}, format='5')
-    vals = loadmat(stream)
-    assert_equal(vals['oned'].shape, (5,1))
-    # Current 4 behavior is 1D -> row vector
-    stream = BytesIO()
-    savemat(stream, {'oned':arr}, format='4')
-    vals = loadmat(stream)
-    assert_equal(vals['oned'].shape, (1, 5))
-    for format in ('4', '5'):
-        # can be explicitly 'column' for oned_as
-        stream = BytesIO()
-        savemat(stream, {'oned':arr},
-                format=format,
-                oned_as='column')
+    warn_ctx = WarningManager()
+    warn_ctx.__enter__()
+    try:
+        # silence warnings for tests
+        warnings.simplefilter('ignore')
+        savemat(stream, {'oned':arr}, format='5')
         vals = loadmat(stream)
         assert_equal(vals['oned'].shape, (5,1))
-        # but different from 'row'
+        # Current 4 behavior is 1D -> row vector
         stream = BytesIO()
-        savemat(stream, {'oned':arr},
-                format=format,
-                oned_as='row')
+        savemat(stream, {'oned':arr}, format='4')
         vals = loadmat(stream)
-        assert_equal(vals['oned'].shape, (1,5))
-    warnings.resetwarnings()
+        assert_equal(vals['oned'].shape, (1, 5))
+        for format in ('4', '5'):
+            # can be explicitly 'column' for oned_as
+            stream = BytesIO()
+            savemat(stream, {'oned':arr},
+                    format=format,
+                    oned_as='column')
+            vals = loadmat(stream)
+            assert_equal(vals['oned'].shape, (5,1))
+            # but different from 'row'
+            stream = BytesIO()
+            savemat(stream, {'oned':arr},
+                    format=format,
+                    oned_as='row')
+            vals = loadmat(stream)
+            assert_equal(vals['oned'].shape, (1,5))
+    finally:
+        warn_ctx.__exit__()
 
 
 def test_compression():
@@ -607,6 +630,7 @@ def test_skip_variable():
     #
     d = factory.get_variables('second')
     yield assert_true, d.has_key('second')
+    factory.mat_stream.close()
 
 
 def test_empty_struct():
@@ -710,8 +734,10 @@ def test_read_opts():
 def test_empty_string():
     # make sure reading empty string does not raise error
     estring_fname = pjoin(test_data_path, 'single_empty_string.mat')
-    rdr = MatFile5Reader_future(open(estring_fname, 'rb'))
+    fp = open(estring_fname, 'rb')
+    rdr = MatFile5Reader_future(fp)
     d = rdr.get_variables()
+    fp.close()
     assert_array_equal(d['a'], np.array([], dtype='U1'))
     # empty string round trip.  Matlab cannot distiguish
     # between a string array that is empty, and a string array
@@ -729,28 +755,36 @@ def test_empty_string():
     rdr = MatFile5Reader_future(stream)
     d = rdr.get_variables()
     assert_array_equal(d['a'], np.array([], dtype='U1'))
+    stream.close()
 
 
 def test_mat4_3d():
     # test behavior when writing 3D arrays to matlab 4 files
     stream = BytesIO()
     arr = np.arange(24).reshape((2,3,4))
-    warnings.simplefilter('error')
-    assert_raises(DeprecationWarning, savemat_future,
-                  stream, {'a': arr}, True, '4')
-    warnings.resetwarnings()
-    # For now, we save a 3D array as 2D
-    warnings.simplefilter('ignore')
-    savemat_future(stream, {'a': arr}, format='4')
-    warnings.resetwarnings()
+
+    warn_ctx = WarningManager()
+    warn_ctx.__enter__()
+    try:
+        warnings.simplefilter('error')
+        assert_raises(DeprecationWarning, savemat_future,
+                      stream, {'a': arr}, True, '4')
+        # For now, we save a 3D array as 2D
+        warnings.simplefilter('ignore')
+        savemat_future(stream, {'a': arr}, format='4')
+    finally:
+        warn_ctx.__exit__()
+
     d = loadmat(stream)
     assert_array_equal(d['a'], arr.reshape((6,4)))
 
 
 def test_func_read():
     func_eg = pjoin(test_data_path, 'testfunc_7.4_GLNX86.mat')
-    rdr = MatFile5Reader_future(open(func_eg, 'rb'))
+    fp = open(func_eg, 'rb')
+    rdr = MatFile5Reader_future(fp)
     d = rdr.get_variables()
+    fp.close()
     yield assert_true, isinstance(d['testfunc'], MatlabFunction)
     stream = BytesIO()
     wtr = MatFile5Writer(stream, oned_as='row')
@@ -759,11 +793,16 @@ def test_func_read():
 
 def test_mat_dtype():
     double_eg = pjoin(test_data_path, 'testmatrix_6.1_SOL2.mat')
-    rdr = MatFile5Reader_future(open(double_eg, 'rb'), mat_dtype=False)
+    fp = open(double_eg, 'rb')
+    rdr = MatFile5Reader_future(fp, mat_dtype=False)
     d = rdr.get_variables()
+    fp.close()
     yield assert_equal, d['testmatrix'].dtype.kind, 'u'
-    rdr = MatFile5Reader_future(open(double_eg, 'rb'), mat_dtype=True)
+
+    fp = open(double_eg, 'rb')
+    rdr = MatFile5Reader_future(fp, mat_dtype=True)
     d = rdr.get_variables()
+    fp.close()
     yield assert_equal, d['testmatrix'].dtype.kind, 'f'
 
 
@@ -790,6 +829,16 @@ def test_mat_struct_squeeze():
                     )
 
 
+def test_scalar_squeeze():
+    stream = BytesIO()
+    in_d = {'scalar': [[0.1]], 'string': 'my name', 'st':{'one':1, 'two':2}}
+    savemat_future(stream, in_d)
+    out_d = loadmat(stream, squeeze_me=True)
+    assert_true(isinstance(out_d['scalar'], float))
+    assert_true(isinstance(out_d['string'], basestring))
+    assert_true(isinstance(out_d['st'], np.ndarray))
+
+
 def test_str_round():
     # from report by Angus McMorland on mailing list 3 May 2010
     stream = BytesIO()
@@ -878,10 +927,20 @@ def test_varmats_from_mat():
 def test_one_by_zero():
     ''' Test 1x0 chars get read correctly '''
     func_eg = pjoin(test_data_path, 'one_by_zero_char.mat')
-    rdr = MatFile5Reader_future(open(func_eg, 'rb'))
+    fp = open(func_eg, 'rb')
+    rdr = MatFile5Reader_future(fp)
     d = rdr.get_variables()
+    fp.close()
     assert_equal(d['var'].shape, (0,))
 
 
+def test_load_mat4_le():
+    # We were getting byte order wrong when reading little-endian floa64 dense
+    # matrices on big-endian platforms
+    mat4_fname = pjoin(test_data_path, 'test_mat4_le_floats.mat')
+    vars = loadmat(mat4_fname)
+    assert_array_equal(vars['a'], [[0.1, 1.2]])
+
+
 if __name__ == "__main__":
     run_module_suite()
