@@ -10,8 +10,8 @@ import numpy as np
 from base import spmatrix, isspmatrix, SparseEfficiencyWarning
 from data import _data_matrix
 import sparsetools
-from sputils import upcast, to_native, isdense, isshape, getdtype, \
-        isscalarlike, isintlike
+from sputils import upcast, upcast_char, to_native, isdense, isshape, \
+     getdtype, isscalarlike, isintlike
 
 
 class _cs_matrix(_data_matrix):
@@ -169,9 +169,11 @@ class _cs_matrix(_data_matrix):
     def __add__(self,other):
         # First check if argument is a scalar
         if isscalarlike(other):
-            # Now we would add this scalar to every element.
-            raise NotImplementedError('adding a scalar to a CSC or CSR '
-                                        'matrix is not supported')
+            if other == 0:
+                return self.copy()
+            else: # Now we would add this scalar to every element.
+                raise NotImplementedError('adding a nonzero scalar to a '
+                                          'sparse matrix is not supported')
         elif isspmatrix(other):
             if (other.shape != self.shape):
                 raise ValueError("inconsistent shapes")
@@ -189,9 +191,11 @@ class _cs_matrix(_data_matrix):
     def __sub__(self,other):
         # First check if argument is a scalar
         if isscalarlike(other):
-            # Now we would add this scalar to every element.
-            raise NotImplementedError('adding a scalar to a sparse '
-                                        'matrix is not supported')
+            if other == 0:
+                return self.copy()
+            else: # Now we would add this scalar to every element.
+                raise NotImplementedError('adding a nonzero scalar to a '
+                                          'sparse matrix is not supported')
         elif isspmatrix(other):
             if (other.shape != self.shape):
                 raise ValueError("inconsistent shapes")
@@ -206,9 +210,11 @@ class _cs_matrix(_data_matrix):
     def __rsub__(self,other):  # other - self
         #note: this can't be replaced by other + (-self) for unsigned types
         if isscalarlike(other):
-            # Now we would add this scalar to every element.
-            raise NotImplementedError('adding a scalar to a sparse '
-                                        'matrix is not supported')
+            if other == 0:
+                return -self.copy()
+            else: # Now we would add this scalar to every element.
+                raise NotImplementedError('adding a nonzero scalar to a '
+                                          'sparse matrix is not supported')
         elif isdense(other):
             # Convert this matrix to a dense matrix and subtract them
             return other - self.todense()
@@ -250,8 +256,9 @@ class _cs_matrix(_data_matrix):
     def _mul_vector(self, other):
         M,N = self.shape
 
-        #output array
-        result = np.zeros( self.shape[0], dtype=upcast(self.dtype,other.dtype) )
+        # output array
+        result = np.zeros(M, dtype=upcast_char(self.dtype.char,
+                                               other.dtype.char))
 
         # csr_matvec or csc_matvec
         fn = getattr(sparsetools,self.format + '_matvec')
@@ -264,7 +271,8 @@ class _cs_matrix(_data_matrix):
         M,N = self.shape
         n_vecs = other.shape[1] #number of column vectors
 
-        result = np.zeros( (M,n_vecs), dtype=upcast(self.dtype,other.dtype) )
+        result = np.zeros((M,n_vecs), dtype=upcast_char(self.dtype.char,
+                                                        other.dtype.char))
 
         # csr_matvecs or csc_matvecs
         fn = getattr(sparsetools,self.format + '_matvecs')
@@ -546,8 +554,9 @@ class _cs_matrix(_data_matrix):
         from coo import coo_matrix
         return coo_matrix((data,(row,col)), self.shape)
 
-    def toarray(self):
-        return self.tocoo(copy=False).toarray()
+    def toarray(self, order=None, out=None):
+        """See the docstring for `spmatrix.toarray`."""
+        return self.tocoo(copy=False).toarray(order=order, out=out)
 
     ##############################################################
     # methods that examine or modify the internal data structure #
