@@ -6,27 +6,40 @@ import _iterative
 import numpy as np
 
 from scipy.sparse.linalg.interface import LinearOperator
+from scipy.lib.decorator import decorator
 from utils import make_system
 
 _type_conv = {'f':'s', 'd':'d', 'F':'c', 'D':'z'}
 
 
 # Part of the docstring common to all iterative solvers
-common_doc = \
+common_doc1 = \
 """
 Parameters
 ----------
-A : {sparse matrix, dense matrix, LinearOperator}
-    The N-by-N matrix of the linear system.
-b : {array, matrix}
+A : {sparse matrix, dense matrix, LinearOperator}"""
+
+common_doc2 = \
+"""b : {array, matrix}
     Right hand side of the linear system. Has shape (N,) or (N,1).
 
-Optional Parameters
--------------------
+Returns
+-------
+x : {array, matrix}
+    The converged solution.
+info : integer
+    Provides convergence information:
+        0  : successful exit
+        >0 : convergence to tolerance not achieved, number of iterations
+        <0 : illegal input or breakdown
+
+Other Parameters
+----------------
 x0  : {array, matrix}
     Starting guess for the solution.
 tol : float
-    Relative tolerance to achieve before terminating.
+    Tolerance to achieve. The algorithm terminates when either the relative
+    or the absolute residual is below `tol`.
 maxiter : integer
     Maximum number of iterations.  Iteration will stop after maxiter
     steps even if the specified tolerance has not been achieved.
@@ -38,38 +51,43 @@ M : {sparse matrix, dense matrix, LinearOperator}
 callback : function
     User-supplied function to call after each iteration.  It is called
     as callback(xk), where xk is the current solution vector.
-
-Outputs
--------
-x : {array, matrix}
-    The converged solution.
-info : integer
-    Provides convergence information:
-        0  : successful exit
-        >0 : convergence to tolerance not achieved, number of iterations
-        <0 : illegal input or breakdown
-
-Deprecated Parameters
-----------------------
 xtype : {'f','d','F','D'}
+    This parameter is deprecated -- avoid using it.
+
     The type of the result.  If None, then it will be determined from
     A.dtype.char and b.  If A does not have a typecode method then it
     will compute A.matvec(x0) to get a typecode.   To save the extra
     computation when A does not have a typecode attribute use xtype=0
     for the same type as b or use xtype='f','d','F',or 'D'.
     This parameter has been superceeded by LinearOperator.
+
 """
 
 
-def set_docstring(header, footer):
+def set_docstring(header, Ainfo, footer=''):
     def combine(fn):
-        fn.__doc__ = header + '\n' + common_doc + '\n' + footer
+        fn.__doc__ = '\n'.join((header, common_doc1,
+                               '    ' + Ainfo.replace('\n', '\n    '),
+                               common_doc2, footer))
         return fn
     return combine
 
+@decorator
+def non_reentrant(func, *a, **kw):
+    d = func.__dict__
+    if d.get('__entered'):
+        raise RuntimeError("%s is not re-entrant" % func.__name__)
+    try:
+        d['__entered'] = True
+        return func(*a, **kw)
+    finally:
+        d['__entered'] = False
 
-
-@set_docstring('Use BIConjugate Gradient iteration to solve A x = b','')
+@set_docstring('Use BIConjugate Gradient iteration to solve A x = b',
+               'The real or complex N-by-N matrix of the linear system\n'
+               'It is required that the linear operator can produce\n'
+               '``Ax`` and ``A^T x``.')
+@non_reentrant
 def bicg(A, b, x0=None, tol=1e-5, maxiter=None, xtype=None, M=None, callback=None):
     A,M,x,b,postprocess = make_system(A,M,x0,b,xtype)
 
@@ -130,7 +148,10 @@ def bicg(A, b, x0=None, tol=1e-5, maxiter=None, xtype=None, M=None, callback=Non
 
     return postprocess(x), info
 
-@set_docstring('Use BIConjugate Gradient STABilized iteration to solve A x = b','')
+@set_docstring('Use BIConjugate Gradient STABilized iteration to solve A x = b',
+               'The real or complex N-by-N matrix of the linear system\n'
+               '``A`` must represent a hermitian, positive definite matrix')
+@non_reentrant
 def bicgstab(A, b, x0=None, tol=1e-5, maxiter=None, xtype=None, M=None, callback=None):
     A,M,x,b,postprocess = make_system(A,M,x0,b,xtype)
 
@@ -166,8 +187,6 @@ def bicgstab(A, b, x0=None, tol=1e-5, maxiter=None, xtype=None, M=None, callback
                 callback(x)
             break
         elif (ijob == 1):
-            if matvec is None:
-                matvec = get_matvec(A)
             work[slice2] *= sclr2
             work[slice2] += sclr1*matvec(work[slice1])
         elif (ijob == 2):
@@ -175,8 +194,6 @@ def bicgstab(A, b, x0=None, tol=1e-5, maxiter=None, xtype=None, M=None, callback
                 psolve = get_psolve(A)
             work[slice1] = psolve(work[slice2])
         elif (ijob == 3):
-            if matvec is None:
-                matvec = get_matvec(A)
             work[slice2] *= sclr2
             work[slice2] += sclr1*matvec(x)
         elif (ijob == 4):
@@ -192,7 +209,10 @@ def bicgstab(A, b, x0=None, tol=1e-5, maxiter=None, xtype=None, M=None, callback
 
     return postprocess(x), info
 
-@set_docstring('Use Conjugate Gradient iteration to solve A x = b','')
+@set_docstring('Use Conjugate Gradient iteration to solve A x = b',
+               'The real or complex N-by-N matrix of the linear system\n'
+               '``A`` must represent a hermitian, positive definite matrix')
+@non_reentrant
 def cg(A, b, x0=None, tol=1e-5, maxiter=None, xtype=None, M=None, callback=None):
     A,M,x,b,postprocess = make_system(A,M,x0,b,xtype)
 
@@ -250,7 +270,9 @@ def cg(A, b, x0=None, tol=1e-5, maxiter=None, xtype=None, M=None, callback=None)
     return postprocess(x), info
 
 
-@set_docstring('Use Conjugate Gradient Squared iteration to solve A x = b','')
+@set_docstring('Use Conjugate Gradient Squared iteration to solve A x = b',
+               'The real-valued N-by-N matrix of the linear system')
+@non_reentrant
 def cgs(A, b, x0=None, tol=1e-5, maxiter=None, xtype=None, M=None, callback=None):
     A,M,x,b,postprocess = make_system(A,M,x0,b,xtype)
 
@@ -306,69 +328,96 @@ def cgs(A, b, x0=None, tol=1e-5, maxiter=None, xtype=None, M=None, callback=None
 
     return postprocess(x), info
 
-
-def gmres(A, b, x0=None, tol=1e-5, restrt=20, maxiter=None, xtype=None, M=None, callback=None):
-    """Use Generalized Minimal RESidual iteration to solve A x = b
+@non_reentrant
+def gmres(A, b, x0=None, tol=1e-5, restart=None, maxiter=None, xtype=None, M=None, callback=None, restrt=None):
+    """
+    Use Generalized Minimal RESidual iteration to solve A x = b.
 
     Parameters
     ----------
     A : {sparse matrix, dense matrix, LinearOperator}
-        The N-by-N matrix of the linear system.
+        The real or complex N-by-N matrix of the linear system.
     b : {array, matrix}
         Right hand side of the linear system. Has shape (N,) or (N,1).
 
-    Optional Parameters
-    -------------------
-    x0  : {array, matrix}
-        Starting guess for the solution.
-    tol : float
-        Relative tolerance to achieve before terminating.
-    restrt : integer
-        Number of iterations between restarts. Larger values increase
-        iteration cost, but may be necessary for convergence.
-    maxiter : integer
-        Maximum number of iterations.  Iteration will stop after maxiter
-        steps even if the specified tolerance has not been achieved.
-    M : {sparse matrix, dense matrix, LinearOperator}
-        Preconditioner for A.  The preconditioner should approximate the
-        inverse of A.  Effective preconditioning dramatically improves the
-        rate of convergence, which implies that fewer iterations are needed
-        to reach a given error tolerance.
-    callback : function
-        User-supplied function to call after each iteration.  It is called
-        as callback(rk), where rk is the current residual vector.
-
-    Outputs
+    Returns
     -------
     x : {array, matrix}
         The converged solution.
-    info : integer
+    info : int
         Provides convergence information:
-            0  : successful exit
-            >0 : convergence to tolerance not achieved, number of iterations
-            <0 : illegal input or breakdown
+          * 0  : successful exit
+          * >0 : convergence to tolerance not achieved, number of iterations
+          * <0 : illegal input or breakdown
 
-    See Also
-    --------
-    LinearOperator
-
-    Deprecated Parameters
-    ---------------------
+    Other parameters
+    ----------------
+    x0 : {array, matrix}
+        Starting guess for the solution (a vector of zeros by default).
+    tol : float
+        Tolerance to achieve. The algorithm terminates when either the relative
+        or the absolute residual is below `tol`.
+    restart : int, optional
+        Number of iterations between restarts. Larger values increase
+        iteration cost, but may be necessary for convergence.
+        Default is 20.
+    maxiter : int, optional
+        Maximum number of iterations.  Iteration will stop after maxiter
+        steps even if the specified tolerance has not been achieved.
     xtype : {'f','d','F','D'}
+        This parameter is DEPRECATED --- avoid using it.
+
         The type of the result.  If None, then it will be determined from
         A.dtype.char and b.  If A does not have a typecode method then it
         will compute A.matvec(x0) to get a typecode.   To save the extra
         computation when A does not have a typecode attribute use xtype=0
         for the same type as b or use xtype='f','d','F',or 'D'.
         This parameter has been superceeded by LinearOperator.
+    M : {sparse matrix, dense matrix, LinearOperator}
+        Inverse of the preconditioner of A.  M should approximate the
+        inverse of A and be easy to solve for (see Notes).  Effective
+        preconditioning dramatically improves the rate of convergence,
+        which implies that fewer iterations are needed to reach a given
+        error tolerance.  By default, no preconditioner is used.
+    callback : function
+        User-supplied function to call after each iteration.  It is called
+        as callback(rk), where rk is the current residual vector.
+    restrt : int, optional
+        DEPRECATED - use `restart` instead.
+
+    See Also
+    --------
+    LinearOperator
+
+    Notes
+    -----
+    A preconditioner, P, is chosen such that P is close to A but easy to solve for.
+    The preconditioner parameter required by this routine is ``M = P^-1``.
+    The inverse should preferably not be calculated explicitly.  Rather, use the
+    following template to produce M::
+
+      # Construct a linear operator that computes P^-1 * x.
+      import scipy.sparse.linalg as spla
+      M_x = lambda x: spla.spsolve(P, x)
+      M = spla.LinearOperator((n, n), M_x)
 
     """
+
+    # Change 'restrt' keyword to 'restart'
+    if restrt is None:
+        restrt = restart
+    elif restart is not None:
+        raise ValueError("Cannot specify both restart and restrt keywords. "
+                         "Preferably use 'restart' only.")
+
     A,M,x,b,postprocess = make_system(A,M,x0,b,xtype)
 
     n = len(b)
     if maxiter is None:
         maxiter = n*10
 
+    if restrt is None:
+        restrt = 20
     restrt = min(restrt, n)
 
     matvec = A.matvec
@@ -441,22 +490,36 @@ def gmres(A, b, x0=None, tol=1e-5, restrt=20, maxiter=None, xtype=None, M=None,
     return postprocess(x), info
 
 
+@non_reentrant
 def qmr(A, b, x0=None, tol=1e-5, maxiter=None, xtype=None, M1=None, M2=None, callback=None):
     """Use Quasi-Minimal Residual iteration to solve A x = b
 
     Parameters
     ----------
     A : {sparse matrix, dense matrix, LinearOperator}
-        The N-by-N matrix of the linear system.
+        The real-valued N-by-N matrix of the linear system.
+        It is required that the linear operator can produce
+        ``Ax`` and ``A^T x``.
     b : {array, matrix}
         Right hand side of the linear system. Has shape (N,) or (N,1).
 
-    Optional Parameters
-    -------------------
+    Returns
+    -------
+    x : {array, matrix}
+        The converged solution.
+    info : integer
+        Provides convergence information:
+            0  : successful exit
+            >0 : convergence to tolerance not achieved, number of iterations
+            <0 : illegal input or breakdown
+
+    Other Parameters
+    ----------------
     x0  : {array, matrix}
         Starting guess for the solution.
     tol : float
-        Relative tolerance to achieve before terminating.
+        Tolerance to achieve. The algorithm terminates when either the relative
+        or the absolute residual is below `tol`.
     maxiter : integer
         Maximum number of iterations.  Iteration will stop after maxiter
         steps even if the specified tolerance has not been achieved.
@@ -469,24 +532,9 @@ def qmr(A, b, x0=None, tol=1e-5, maxiter=None, xtype=None, M1=None, M2=None, cal
     callback : function
         User-supplied function to call after each iteration.  It is called
         as callback(xk), where xk is the current solution vector.
-
-    Outputs
-    -------
-    x : {array, matrix}
-        The converged solution.
-    info : integer
-        Provides convergence information:
-            0  : successful exit
-            >0 : convergence to tolerance not achieved, number of iterations
-            <0 : illegal input or breakdown
-
-    See Also
-    --------
-    LinearOperator
-
-    Deprecated Parameters
-    ---------------------
     xtype : {'f','d','F','D'}
+        This parameter is DEPRECATED -- avoid using it.
+
         The type of the result.  If None, then it will be determined from
         A.dtype.char and b.  If A does not have a typecode method then it
         will compute A.matvec(x0) to get a typecode.   To save the extra
@@ -494,6 +542,10 @@ def qmr(A, b, x0=None, tol=1e-5, maxiter=None, xtype=None, M1=None, M2=None, cal
         for the same type as b or use xtype='f','d','F',or 'D'.
         This parameter has been superceeded by LinearOperator.
 
+    See Also
+    --------
+    LinearOperator
+
     """
     A_ = A
     A,M,x,b,postprocess = make_system(A,None,x0,b,xtype)
