@@ -21,7 +21,7 @@ import futil
 from numpy.testing.decorators import setastest
 import warnings
 
-__all__ = ['find_repeats', 'mvsdist',
+__all__ = ['mvsdist',
            'bayes_mvs', 'kstat', 'kstatvar', 'probplot', 'ppcc_max', 'ppcc_plot',
            'boxcox_llf', 'boxcox', 'boxcox_normmax', 'boxcox_normplot',
            'shapiro', 'anderson', 'ansari', 'bartlett', 'levene', 'binom_test',
@@ -30,114 +30,61 @@ __all__ = ['find_repeats', 'mvsdist',
           ]
 
 
-##########################################################
-###  Bayesian confidence intervals for mean, variance, std
-##########################################################
-
-##  See the paper "A Bayesian perspective on estimating
-##    mean, variance, and standard-deviation from data" by
-##    Travis E. Oliphant
-##    at http://dspace.byu.edu/bitstream/1877/438/1/bayes_mvs.pdf
-##    (Permanent link at http://hdl.handle.net/1877/438 )
-
-# assume distributions are gaussian with given means and variances.
-def _gauss_mvs(x, n, alpha):
-    xbar = x.mean()
-    C = x.var()
-    val = distributions.norm.ppf((1+alpha)/2.0)
-    # mean is a Gaussian with mean xbar and variance C/n
-    mp = xbar
-    fac0 = sqrt(C/n)
-    term = fac0*val
-    ma = mp - term
-    mb = mp + term
-    # var is a Gaussian with mean C and variance 2*C*C/n
-    vp = C
-    fac1 = sqrt(2.0/n)*C
-    term = fac1*val
-    va = vp - term
-    vb = vp + term
-    # std is a Gaussian with mean sqrt(C) and variance C/(2*n)
-    st = sqrt(C)
-    fac2 = sqrt(0.5)*fac0
-    term = fac2*val
-    sta = st - term
-    stb = st + term
-    return (mp, (ma, mb)), (vp, (va, vb)), (st, (sta, stb))
-
-
-##  Assumes all is known is that mean, and std (variance,axis=0) exist
-##   and are the same for all the data.  Uses Jeffrey's prior
-##
-##  Returns alpha confidence interval for the mean, variance,
-##      and std.
-
-def bayes_mvs(data,alpha=0.90):
-    """Return Bayesian confidence intervals for the mean, var, and std.
-
-    Assumes 1-d data all has same mean and variance and uses Jeffrey's prior
-    for variance and std.
-
-    alpha gives the probability that the returned confidence interval contains
-    the true parameter.
-
-    Uses mean of conditional pdf as center estimate
-    (but centers confidence interval on the median)
-
-    Returns (center, (a, b)) for each of mean, variance and standard deviation.
-    Requires 2 or more data-points.
-    """
-    x = ravel(data)
-    n = len(x)
-    assert(n > 1)
-    assert(alpha < 1 and alpha > 0)
-    n = float(n)
-    if (n > 1000): # just a guess.  The curves look very similar at this point.
-        return _gauss_mvs(x, n, alpha)
-    xbar = x.mean()
-    C = x.var()
-    # mean
-    fac = sqrt(C/(n-1))
-    tval = distributions.t.ppf((1+alpha)/2.0,n-1)
-    delta = fac*tval
-    ma = xbar - delta
-    mb = xbar + delta
-    mp = xbar
-    # var
-    fac = n*C/2.0
-    a = (n-1)/2
-    if (n < 4):
-        peak = distributions.invgamma.ppf(0.5,a)
-    else:
-        peak = 2.0/(n-3.0)
-    q1 = (1-alpha)/2.0
-    q2 = (1+alpha)/2.0
-    va = fac*distributions.invgamma.ppf(q1,a)
-    vb = fac*distributions.invgamma.ppf(q2,a)
-    vp = peak*fac
-    # std
-    fac = sqrt(fac)
-    if (n < 3):
-        peak = distributions.gengamma.ppf(0.5,a,-2)
-        stp = fac*peak
-    else:
-        ndiv2 = (n-1)/2.0
-        term = special.gammaln(ndiv2-0.5)-special.gammaln(ndiv2)
-        term += (log(n)+log(C)-log(2.0))*0.5
-        stp = exp(term)
-    q1 = (1-alpha)/2.0
-    q2 = (1+alpha)/2.0
-    sta = fac*distributions.gengamma.ppf(q1,a,-2)
-    stb = fac*distributions.gengamma.ppf(q2,a,-2)
-
-    return (mp,(ma,mb)),(vp,(va,vb)),(stp,(sta,stb))
-
-def mvsdist(data):
-    """Return 'frozen' distributions for mean, variance, and standard deviation of data.
+def bayes_mvs(data, alpha=0.90):
+    """Bayesian confidence intervals for the mean, var, and std.
 
     Parameters
     ----------
-    data : array-like (raveled to 1-d)
+    data : array_like
+       Input data, if multi-dimensional it is flattened to 1-D by `bayes_mvs`.
+       Requires 2 or more data points.
+    alpha : float, optional
+       Probability that the returned confidence interval contains
+       the true parameter.
+
+    Returns
+    -------
+    Returns a 3 output arguments for each of mean, variance, and standard deviation.
+       Each of the outputs is a pair:
+          (center, (lower, upper))
+       with center the mean of the conditional pdf of the value given the data
+       and (lower, upper) is a confidence interval centered on the median,
+       containing the estimate to a probability alpha.
+
+    mctr, (ma, mb) :
+       Estimates for mean
+    vctr, (va, vb) :
+       Estimates for variance
+    sctr, (sa, sb) :
+       Estimates for standard deviation
+
+    Notes
+    -----
+    Converts data to 1-D and assumes all data has the same mean and variance.
+    Uses Jeffrey's prior for variance and std.
+
+    Equivalent to tuple((x.mean(), x.interval(alpha)) for x in mvsdist(dat))
+
+    References
+    ----------
+    T.E. Oliphant, "A Bayesian perspective on estimating mean, variance, and
+    standard-deviation from data", http://hdl.handle.net/1877/438, 2006.
+
+    """
+    res = mvsdist(data)
+    if alpha >= 1 or alpha <= 0:
+        raise ValueError("0 < alpha < 1 is required, but alpha=%s was given." % alpha)
+    return tuple((x.mean(), x.interval(alpha)) for x in res)
+
+def mvsdist(data):
+    """
+    'Frozen' distributions for mean, variance, and standard deviation of data.
+
+    Parameters
+    ----------
+    data : array_like
+        Input array. Converted to 1-D using ravel.
+        Requires 2 or more data-points.
 
     Returns
     -------
@@ -147,6 +94,32 @@ def mvsdist(data):
         Distribution object representing the variance of the data
     sdist : "frozen" distribution object
         Distribution object representing the standard deviation of the data
+
+    Notes
+    -----
+    The return values from bayes_mvs(data) is equivalent to
+    ``tuple((x.mean(), x.interval(0.90)) for x in mvsdist(data))``.
+
+    In other words, calling ``<dist>.mean()`` and ``<dist>.interval(0.90)``
+    on the three distribution objects returned from this function will give
+    the same results that are returned from `bayes_mvs`.
+
+    Examples
+    --------
+    >>> from scipy.stats import mvsdist
+    >>> data = [6, 9, 12, 7, 8, 8, 13]
+    >>> mean, var, std = mvsdist(data)
+
+    We now have frozen distribution objects "mean", "var" and "std" that we can
+    examine:
+
+    >>> mean.mean()
+    9.0
+    >>> mean.interval(0.95)
+    (6.6120585482655692, 11.387941451734431)
+    >>> mean.std()
+    1.1952286093343936
+
     """
     x = ravel(data)
     n = len(x)
@@ -168,32 +141,6 @@ def mvsdist(data):
     return mdist, vdist, sdist
 
 
-################################
-##  K-Statistic
-################################
-
-###
-## The n-th k-statistic is the unique symmetric unbiased estimator of
-##   the n-th cumulant kappa_n
-## The cumulants are related to central moments but are specifically defined
-##   using a power series expansion of the logarithm of the characteristic
-##   function (which is the Fourier transform of the PDF).
-##   In particular let phi(t) be the characteristic function, then
-##                     _
-##         ln phi(t) = > kappa_n (it)^n / n!    (sum from n=0 to inf)
-##                     -
-##
-##   The first few cumulants (kappa_n)  in terms of central moments (mu_n) are
-##    kappa_1 = mu_1
-##    kappa_2 = mu_2
-##    kappa_3 = mu_3
-##    kappa_4 = mu_4 - 3*mu_2**2
-##    kappa_5 = mu_5 - 10*mu_2 * mu_3
-##
-## Source:  http://mathworld.wolfram.com/Cumulant.html
-##          http://mathworld.wolfram.com/k-Statistic.html
-##
-
 def kstat(data,n=2):
     """
     Return the nth k-statistic (1<=n<=4 so far).
@@ -213,6 +160,33 @@ def kstat(data,n=2):
     kstat : float
         The nth k-statistic.
 
+    See Also
+    --------
+    kstatvar: Returns an unbiased estimator of the variance of the k-statistic.
+
+    Notes
+    -----
+    The cumulants are related to central moments but are specifically defined
+    using a power series expansion of the logarithm of the characteristic
+    function (which is the Fourier transform of the PDF).
+    In particular let phi(t) be the characteristic function, then::
+
+        ln phi(t) = > kappa_n (it)^n / n!    (sum from n=0 to inf)
+
+    The first few cumulants (kappa_n)  in terms of central moments (mu_n) are::
+
+        kappa_1 = mu_1
+        kappa_2 = mu_2
+        kappa_3 = mu_3
+        kappa_4 = mu_4 - 3*mu_2**2
+        kappa_5 = mu_5 - 10*mu_2 * mu_3
+
+    References
+    ----------
+    http://mathworld.wolfram.com/k-Statistic.html
+
+    http://mathworld.wolfram.com/Cumulant.html
+
     """
     if n > 4 or n < 1:
         raise ValueError("k-statistics only supported for 1<=n<=4")
@@ -239,6 +213,8 @@ def kstatvar(data,n=2):
     """
     Returns an unbiased estimator of the variance of the k-statistic.
 
+    See `kstat` for more details of the k-statistic.
+
     Parameters
     ----------
     data : array_like
@@ -251,6 +227,10 @@ def kstatvar(data,n=2):
     kstatvar : float
         The nth k-statistic variance.
 
+    See Also
+    --------
+    kstat
+
     """
     data = ravel(data)
     N = len(data)
@@ -264,25 +244,87 @@ def kstatvar(data,n=2):
         raise ValueError("Only n=1 or n=2 supported.")
 
 
-#__all__ = ['probplot','ppcc_max','ppcc_plot','boxcox','boxcox_llf',
-#           'boxcox_normplot','boxcox_normmax','shapiro']
+def probplot(x, sparams=(), dist='norm', fit=True, plot=None):
+    """
+    Calculate quantiles for a probability plot of sample data against a
+    specified theoretical distribution.
 
-def probplot(x, sparams=(), dist='norm', fit=1, plot=None):
-    """Return (osm, osr){,(scale,loc,r)} where (osm, osr) are order statistic
-    medians and ordered response data respectively so that plot(osm, osr)
-    is a probability plot.  If fit==1, then do a regression fit and compute the
-    slope (scale), intercept (loc), and correlation coefficient (r), of the
-    best straight line through the points.  If fit==0, only (osm, osr) is
-    returned.
+    `probplot` optionally calculates a best-fit line for the data and plots the
+    results using Matplotlib or a given plot function.
+
+    Parameters
+    ----------
+    x : array_like
+        Sample/response data from which `probplot` creates the plot.
+    sparams : tuple, optional
+        Distribution-specific shape parameters (location(s) and scale(s)).
+    dist : str, optional
+        Distribution function name. The default is 'norm' for a normal
+        probability plot.
+    fit : bool, optional
+        Fit a least-squares regression (best-fit) line to the sample data if
+        True (default).
+    plot : object, optional
+        If given, plots the quantiles and least squares fit.
+        `plot` is an object with methods "plot", "title", "xlabel", "ylabel"
+        and "text". The matplotlib.pyplot module or a Matplotlib axes object can
+        be used, or a custom object with the same methods.
+        By default, no plot is created.
+
+    Notes
+    -----
+    Even if `plot` is given, the figure is not shown or saved by `probplot`;
+    ``plot.show()`` or ``plot.savefig('figname.png')`` should be used after
+    calling `probplot`.
+
+    Returns
+    -------
+    (osm, osr) : tuple of ndarrays
+        Tuple of theoretical quantiles (osm, or order statistic medians) and
+        ordered responses (osr).
+    (slope, intercept, r) : tuple of floats, optional
+        Tuple  containing the result of the least-squares fit, if that is
+        performed by `probplot`. `r` is the square root of the coefficient of
+        determination.  If ``fit=False`` and ``plot=None``, this tuple is not
+        returned.
+
+    Examples
+    --------
+    >>> import scipy.stats as stats
+    >>> nsample = 100
+    >>> np.random.seed(7654321)
+
+    A t distribution with small degrees of freedom:
+
+    >>> ax1 = plt.subplot(221)
+    >>> x = stats.t.rvs(3, size=nsample)
+    >>> res = stats.probplot(x, plot=plt)
+
+    A t distribution with larger degrees of freedom:
+
+    >>> ax2 = plt.subplot(222)
+    >>> x = stats.t.rvs(25, size=nsample)
+    >>> res = stats.probplot(x, plot=plt)
+
+    A mixture of 2 normal distributions with broadcasting:
+
+    >>> ax3 = plt.subplot(223)
+    >>> x = stats.norm.rvs(loc=[0,5], scale=[1,1.5], size=(nsample/2.,2)).ravel()
+    >>> res = stats.probplot(x, plot=plt)
+
+    A standard normal distribution:
+
+    >>> ax4 = plt.subplot(224)
+    >>> x = stats.norm.rvs(loc=0, scale=1, size=nsample)
+    >>> res = stats.probplot(x, plot=plt)
 
-    sparams is a tuple of shape parameter arguments for the distribution.
     """
     N = len(x)
-    Ui = zeros(N)*1.0
-    Ui[-1] = 0.5**(1.0/N)
-    Ui[0] = 1-Ui[-1]
-    i = arange(2,N)
-    Ui[1:-1] = (i-0.3175)/(N+0.365)
+    Ui = zeros(N) * 1.0
+    Ui[-1] = 0.5**(1.0 /N)
+    Ui[0] = 1 - Ui[-1]
+    i = arange(2, N)
+    Ui[1:-1] = (i - 0.3175) / (N + 0.365)
     try:
         ppf_func = eval('distributions.%s.ppf' % dist)
     except AttributeError:
@@ -291,7 +333,7 @@ def probplot(x, sparams=(), dist='norm', fit=1, plot=None):
         sparams = ()
     if isscalar(sparams):
         sparams = (sparams,)
-    if not isinstance(sparams,types.TupleType):
+    if not isinstance(sparams, types.TupleType):
         sparams = tuple(sparams)
     """
     res = inspect.getargspec(ppf_func)
@@ -303,22 +345,24 @@ def probplot(x, sparams=(), dist='norm', fit=1, plot=None):
        (len(sparams) > len(res[0])-3):
         raise ValueError("Incorrect number of shape parameters.")
     """
-    osm = ppf_func(Ui,*sparams)
+    osm = ppf_func(Ui, *sparams)
     osr = sort(x)
     if fit or (plot is not None):
         # perform a linear fit.
-        slope, intercept, r, prob, sterrest = stats.linregress(osm,osr)
+        slope, intercept, r, prob, sterrest = stats.linregress(osm, osr)
     if plot is not None:
         plot.plot(osm, osr, 'o', osm, slope*osm + intercept)
         plot.title('Probability Plot')
-        plot.xlabel('Order Statistic Medians')
+        plot.xlabel('Quantiles')
         plot.ylabel('Ordered Values')
 
-        xmin,xmax= amin(osm),amax(osm)
-        ymin,ymax= amin(x),amax(x)
-        posx,posy = xmin+0.70*(xmax-xmin), ymin+0.01*(ymax-ymin)
-        #plot.addtext("r^2^=%1.4f" % r, xy=pos,tosys=1)
-        plot.text(posx,posy, "r^2=%1.4f" % r)
+        xmin = amin(osm)
+        xmax = amax(osm)
+        ymin = amin(x)
+        ymax = amax(x)
+        posx = xmin + 0.70 * (xmax - xmin)
+        posy = ymin + 0.01 * (ymax - ymin)
+        plot.text(posx, posy, "r^2=%1.4f" % r)
     if fit:
         return (osm, osr), (slope, intercept, r)
     else:
@@ -488,7 +532,7 @@ def boxcox_normplot(x,la,lb,plot=None,N=80):
         plot.ylabel('Transformation parameter')
     return svals, ppcc
 
-def shapiro(x,a=None,reta=0):
+def shapiro(x,a=None,reta=False):
     """
     Perform the Shapiro-Wilk test for normality.
 
@@ -498,21 +542,21 @@ def shapiro(x,a=None,reta=0):
     Parameters
     ----------
     x : array_like
-        array of sample data
+        Array of sample data.
     a : array_like, optional
-        array of internal parameters used in the calculation.  If these
+        Array of internal parameters used in the calculation.  If these
         are not given, they will be computed internally.  If x has length
         n, then a must have length n/2.
-    reta : {True, False}
-        whether or not to return the internally computed a values.  The
+    reta : bool, optional
+        Whether or not to return the internally computed a values.  The
         default is False.
 
     Returns
     -------
     W : float
-        The test statistic
+        The test statistic.
     p-value : float
-        The p-value for the hypothesis test
+        The p-value for the hypothesis test.
     a : array_like, optional
         If `reta` is True, then these are the internally computed "a"
         values that may be passed into this function on future calls.
@@ -533,10 +577,11 @@ def shapiro(x,a=None,reta=0):
         a = zeros(N,'f')
         init = 0
     else:
-        assert(len(a) == N/2), "a must be == len(x)/2"
+        if len(a) != N//2:
+            raise ValueError("len(a) must equal len(x)/2")
         init = 1
     y = sort(x)
-    a,w,pw,ifault = statlib.swilk(y,a[:N/2],init)
+    a, w, pw, ifault = statlib.swilk(y, a[:N//2], init)
     if not ifault in [0,2]:
         warnings.warn(str(ifault))
     if N > 5000:
@@ -782,13 +827,12 @@ def bartlett(*args):
     Returns
     -------
     T : float
-        the test statistic
+        The test statistic.
     p-value : float
-        the p-value of the test
+        The p-value of the test.
 
     References
     ----------
-
     .. [1]  http://www.itl.nist.gov/div898/handbook/eda/section3/eda357.htm
 
     .. [2]  Snedecor, George W. and Cochran, William G. (1989), Statistical
@@ -814,11 +858,11 @@ def bartlett(*args):
 
 def levene(*args,**kwds):
     """
-    Perform Levene test for equal variances
+    Perform Levene test for equal variances.
 
     The Levene test tests the null hypothesis that all input samples
     are from populations with equal variances.  Levene's test is an
-    alternative to Bartlett's test `bartlett`_ in the case where
+    alternative to Bartlett's test `bartlett` in the case where
     there are significant deviations from normality.
 
     Parameters
@@ -836,21 +880,18 @@ def levene(*args,**kwds):
     Returns
     -------
     W : float
-        the test statistic
+        The test statistic.
     p-value : float
-        the p-value for the test
+        The p-value for the test.
 
     Notes
     -----
     Three variations of Levene's test are possible.  The possibilities
     and their recommended usages are:
 
-    'median'
-       Recommended for skewed (non-normal) distributions
-    'mean'
-       Recommended for symmetric, moderate-tailed distributions
-    'trimmed'
-       Recommended for heavy-tailed distributions
+      * 'median' : Recommended for skewed (non-normal) distributions>
+      * 'mean' : Recommended for symmetric, moderate-tailed distributions.
+      * 'trimmed' : Recommended for heavy-tailed distributions.
 
     References
     ----------
@@ -992,12 +1033,12 @@ def _apply_func(x,g,func):
 
 def fligner(*args,**kwds):
     """
-    Perform Fligner's test for equal variances
+    Perform Fligner's test for equal variances.
 
     Fligner's test tests the null hypothesis that all input samples
     are from populations with equal variances.  Fligner's test is
-    non-parametric in contrast to Bartlett's test bartlett_ and
-    Levene's test levene_.
+    non-parametric in contrast to Bartlett's test `bartlett` and
+    Levene's test `levene`.
 
     Parameters
     ----------
@@ -1023,11 +1064,10 @@ def fligner(*args,**kwds):
     -----
     As with Levene's test there are three variants
     of Fligner's test that differ by the measure of central
-    tendency used in the test.  See levene_ for more information.
+    tendency used in the test.  See `levene` for more information.
 
     References
     ----------
-
     .. [1] http://www.stat.psu.edu/~bgl/center/tr/TR993.ps
 
     .. [2] Fligner, M.A. and Killeen, T.J. (1976). Distribution-free two-sample
@@ -1087,7 +1127,7 @@ def fligner(*args,**kwds):
 
 def mood(x,y):
     """
-    Perform Mood's test for equal scale parameters
+    Perform Mood's test for equal scale parameters.
 
     Mood's two-sample test for scale parameters is a non-parametric
     test for the null hypothesis that two samples are drawn from the
@@ -1096,12 +1136,12 @@ def mood(x,y):
     Parameters
     ----------
     x, y : array_like
-        arrays of sample data
+        Arrays of sample data.
 
     Returns
     -------
     p-value : float
-        The p-value for the hypothesis test
+        The p-value for the hypothesis test.
 
     See Also
     --------
@@ -1183,7 +1223,7 @@ def oneway(*args,**kwds):
 
 def wilcoxon(x,y=None):
     """
-    Calculate the Wilcoxon signed-rank test
+    Calculate the Wilcoxon signed-rank test.
 
     The Wilcoxon signed-rank test tests the null hypothesis that two
     related samples come from the same distribution. It is a a
@@ -1192,8 +1232,8 @@ def wilcoxon(x,y=None):
     Parameters
     ----------
     x : array_like
-        The first set of measurements
-    y : array_like, optional, default None
+        The first set of measurements.
+    y : array_like, optional
         The second set of measurements.  If y is not given, then the x array
         is considered to be the differences between the two sets of
         measurements.
@@ -1204,7 +1244,7 @@ def wilcoxon(x,y=None):
         The test statistic under the large-sample approximation that the
         signed-rank statistic is normally distributed.
     p-value : float
-        The two-sided p-value for the test
+        The two-sided p-value for the test.
 
     Notes
     -----
@@ -1281,19 +1321,21 @@ def pdf_fromgamma(g1,g2,g3=0.0,g4=None):
         return totp(xn)*exp(-xn*xn/2.0)
     return thefunc
 
-def circmean(samples, high=2*pi, low=0):
+def circmean(samples, high=2*pi, low=0, axis=None):
     """
-    Compute the circular mean for samples assumed to be in the range
-    [low to high].
+    Compute the circular mean for samples in a range.
 
     Parameters
     ----------
     samples : array_like
         Input array.
+    high : float or int, optional
+        High boundary for circular mean range.  Default is ``2*pi``.
     low : float or int, optional
         Low boundary for circular mean range.  Default is 0.
-    high : float or int, optional
-        High boundary for circular mean range.  Default is 2*pi.
+    axis : int, optional
+        Axis along which means are computed.  The default is to compute
+        the mean of the flattened array.
 
     Returns
     -------
@@ -1302,15 +1344,17 @@ def circmean(samples, high=2*pi, low=0):
 
     """
     ang = (samples - low)*2*pi / (high-low)
-    res = angle(np.mean(exp(1j*ang), axis=0))
-    if (res < 0):
+    res = angle(np.mean(exp(1j*ang), axis=axis))
+    mask = res < 0
+    if (mask.ndim > 0):
+        res[mask] += 2*pi
+    elif mask:
         res = res + 2*pi
     return res*(high-low)/2.0/pi + low
 
-def circvar(samples, high=2*pi, low=0):
+def circvar(samples, high=2*pi, low=0, axis=None):
     """
-    Compute the circular variance for samples assumed to be in the range
-    [low to high].
+    Compute the circular variance for samples assumed to be in a range
 
     Parameters
     ----------
@@ -1319,20 +1363,28 @@ def circvar(samples, high=2*pi, low=0):
     low : float or int, optional
         Low boundary for circular variance range.  Default is 0.
     high : float or int, optional
-        High boundary for circular variance range.  Default is 2*pi.
+        High boundary for circular variance range.  Default is ``2*pi``.
+    axis : int, optional
+        Axis along which variances are computed.  The default is to compute
+        the variance of the flattened array.
 
     Returns
     -------
     circvar : float
         Circular variance.
 
+    Notes
+    -----
+    This uses a definition of circular variance that in the limit of small
+    angles returns a number close to the 'linear' variance.
+
     """
     ang = (samples - low)*2*pi / (high-low)
-    res = np.mean(exp(1j*ang), axis=0)
-    V = 1-abs(res)
-    return ((high-low)/2.0/pi)**2 * V
+    res = np.mean(exp(1j*ang), axis=axis)
+    R = abs(res)
+    return ((high-low)/2.0/pi)**2 * 2 * log(1/R)
 
-def circstd(samples, high=2*pi, low=0):
+def circstd(samples, high=2*pi, low=0, axis=None):
     """
     Compute the circular standard deviation for samples assumed to be in the
     range [low to high].
@@ -1344,18 +1396,27 @@ def circstd(samples, high=2*pi, low=0):
     low : float or int, optional
         Low boundary for circular standard deviation range.  Default is 0.
     high : float or int, optional
-        High boundary for circular standard deviation range.  Default is 2*pi.
+        High boundary for circular standard deviation range.
+        Default is ``2*pi``.
+    axis : int, optional
+        Axis along which standard deviations are computed.  The default is
+        to compute the standard deviation of the flattened array.
 
     Returns
     -------
     circstd : float
         Circular standard deviation.
 
+    Notes
+    -----
+    This uses a definition of circular standard deviation that in the limit of
+    small angles returns a number close to the 'linear' standard deviation.
+
     """
     ang = (samples - low)*2*pi / (high-low)
-    res = np.mean(exp(1j*ang), axis=0)
-    V = 1-abs(res)
-    return ((high-low)/2.0/pi) * sqrt(V)
+    res = np.mean(exp(1j*ang), axis=axis)
+    R = abs(res)
+    return ((high-low)/2.0/pi) * sqrt(-2*log(R))
 
 
 
