@@ -606,6 +606,7 @@ class TestCorrSpearmanrTies(TestCase):
 ### I need to figure out how to do this one.
 
 
+@dec.knownfailureif(sys.version[:3] < '2.5', "Can't index array with np.int64")
 def test_kendalltau():
     """Some tests for kendalltau."""
 
@@ -989,13 +990,13 @@ class TestHMean(TestCase):
         assert_array_almost_equal(actual1, desired1, decimal=14)
 
 
-class TestPercentile(TestCase):
+class TestScoreatpercentile(TestCase):
     def setUp(self):
         self.a1 = [3,4,5,10,-3,-5,6]
         self.a2 = [3,-6,-2,8,7,4,2,1]
         self.a3 = [3.,4,5,10,-3,-5,-6,7.0]
 
-    def test_percentile(self):
+    def test_basic(self):
         x = arange(8) * 0.5
         assert_equal(stats.scoreatpercentile(x, 0), 0.)
         assert_equal(stats.scoreatpercentile(x, 100), 3.5)
@@ -1007,8 +1008,58 @@ class TestPercentile(TestCase):
                    [4, 4, 3],
                    [1, 1, 1],
                    [1, 1, 1]])
-        assert_array_equal(stats.scoreatpercentile(x,50),
-                           [1,1,1])
+        assert_array_equal(stats.scoreatpercentile(x, 50), [1,1,1])
+
+    def test_fraction(self):
+        scoreatperc = stats.scoreatpercentile
+
+        # Test defaults
+        assert_equal(scoreatperc(range(10), 50), 4.5)
+        assert_equal(scoreatperc(range(10), 50, (2,7)), 4.5)
+        assert_equal(scoreatperc(range(100), 50, limit=(1, 8)), 4.5)
+        assert_equal(scoreatperc(np.array([1, 10 ,100]), 50, (10,100)), 55)
+        assert_equal(scoreatperc(np.array([1, 10 ,100]), 50, (1,10)), 5.5)
+
+        # explicitly specify interpolation_method 'fraction' (the default)
+        assert_equal(scoreatperc(range(10), 50, interpolation_method='fraction'),
+                     4.5)
+        assert_equal(scoreatperc(range(10), 50, limit=(2, 7),
+                                 interpolation_method='fraction'),
+                     4.5)
+        assert_equal(scoreatperc(range(100), 50, limit=(1, 8),
+                                 interpolation_method='fraction'),
+                     4.5)
+        assert_equal(scoreatperc(np.array([1, 10 ,100]), 50, (10, 100),
+                                 interpolation_method='fraction'),
+                     55)
+        assert_equal(scoreatperc(np.array([1, 10 ,100]), 50, (1,10),
+                                 interpolation_method='fraction'),
+                     5.5)
+
+    def test_lower_higher(self):
+        scoreatperc = stats.scoreatpercentile
+
+        # interpolation_method 'lower'/'higher'
+        assert_equal(scoreatperc(range(10), 50,
+                                 interpolation_method='lower'), 4)
+        assert_equal(scoreatperc(range(10), 50,
+                                 interpolation_method='higher'), 5)
+        assert_equal(scoreatperc(range(10), 50, (2,7),
+                                 interpolation_method='lower'), 4)
+        assert_equal(scoreatperc(range(10), 50, limit=(2,7),
+                                 interpolation_method='higher'), 5)
+        assert_equal(scoreatperc(range(100), 50, (1,8),
+                                 interpolation_method='lower'), 4)
+        assert_equal(scoreatperc(range(100), 50, (1,8),
+                                 interpolation_method='higher'), 5)
+        assert_equal(scoreatperc(np.array([1, 10 ,100]), 50, (10,100),
+                                 interpolation_method='lower'), 10)
+        assert_equal(scoreatperc(np.array([1, 10 ,100]), 50, limit=(10, 100),
+                                 interpolation_method='higher'), 100)
+        assert_equal(scoreatperc(np.array([1, 10 ,100]), 50, (1,10),
+                                 interpolation_method='lower'), 1)
+        assert_equal(scoreatperc(np.array([1, 10 ,100]), 50, limit=(1,10),
+                                 interpolation_method='higher'), 10)
 
 
 class TestCMedian(TestCase):
@@ -1060,7 +1111,7 @@ class TestVariability(TestCase):
         assert_array_almost_equal(desired,y,decimal=12)
 
     def test_zmap_axis(self):
-        """Test use of 'axis' keyword in zmap."""        
+        """Test use of 'axis' keyword in zmap."""
         x = np.array([[0.0, 0.0, 1.0, 1.0],
                       [1.0, 1.0, 1.0, 2.0],
                       [2.0, 0.0, 2.0, 0.0]])
@@ -1080,7 +1131,7 @@ class TestVariability(TestCase):
                        [1.0, -1.0, 1.0, -1.0]]
 
         assert_array_almost_equal(z0, z0_expected)
-        assert_array_almost_equal(z1, z1_expected)        
+        assert_array_almost_equal(z1, z1_expected)
 
     def test_zmap_ddof(self):
         """Test use of 'ddof' keyword in zmap."""
@@ -1262,17 +1313,6 @@ class TestStudentTest(TestCase):
         assert_array_almost_equal(t, self.T1_2)
         assert_array_almost_equal(p, self.P1_2)
 
-def test_scoreatpercentile():
-    assert_equal(stats.scoreatpercentile(range(10), 50), 4.5)
-    assert_equal(stats.scoreatpercentile(range(10), 50, (2,7)), 4.5)
-    assert_equal(stats.scoreatpercentile(range(100), 50, (1,8)), 4.5)
-
-    assert_equal(stats.scoreatpercentile(np.array([1, 10 ,100]),
-                                         50, (10,100)),
-                 55)
-    assert_equal(stats.scoreatpercentile(np.array([1, 10 ,100]),
-                                         50, (1,10)),
-                 5.5)
 
 def test_percentileofscore():
     pcos = stats.percentileofscore
@@ -1492,7 +1532,7 @@ def test_ttest_rel():
         #test zero division problem
         t,p = stats.ttest_rel([0,0,0],[1,1,1])
         assert_equal((np.abs(t),p), (np.inf, 0))
-        assert_almost_equal(stats.ttest_rel([0,0,0], [0,0,0]), (1.0, 0.42264973081037421))
+        assert_equal(stats.ttest_rel([0,0,0], [0,0,0]), (np.nan, np.nan))
 
         #check that nan in input array result in nan output
         anan = np.array([[1,np.nan],[-1,1]])
@@ -1537,7 +1577,7 @@ def test_ttest_ind():
         #test zero division problem
         t,p = stats.ttest_ind([0,0,0],[1,1,1])
         assert_equal((np.abs(t),p), (np.inf, 0))
-        assert_almost_equal(stats.ttest_ind([0,0,0], [0,0,0]), (1.0, 0.37390096630005898))
+        assert_equal(stats.ttest_ind([0,0,0], [0,0,0]), (np.nan, np.nan))
 
         #check that nan in input array result in nan output
         anan = np.array([[1,np.nan],[-1,1]])
@@ -1545,6 +1585,71 @@ def test_ttest_ind():
     finally:
         np.seterr(**olderr)
 
+def test_ttest_ind_with_uneq_var():
+      
+    # check vs. R
+    a = (1, 2, 3)
+    b = (1.1, 2.9, 4.2)
+    pr = 0.53619490753126731
+    tr = -0.68649512735572582
+    t, p = stats.ttest_ind(a, b, equal_var = False)
+    assert_array_almost_equal([t,p], [tr, pr])
+    
+    a = (1, 2, 3, 4)
+    pr = 0.84354139131608286
+    tr = -0.2108663315950719
+    t, p = stats.ttest_ind(a, b, equal_var = False)
+    assert_array_almost_equal([t,p], [tr, pr])
+
+    #regression test
+    tr = 1.0912746897927283
+    tr_uneq_n = 0.66745638708050492
+    pr = 0.27647831993021388
+    pr_uneq_n = 0.50873585065616544
+    tpr = ([tr,-tr],[pr,pr])
+
+    rvs3 = np.linspace(1,100, 25)
+    rvs2 = np.linspace(1,100,100)
+    rvs1 = np.linspace(5,105,100)
+    rvs1_2D = np.array([rvs1, rvs2])
+    rvs2_2D = np.array([rvs2, rvs1])
+
+    t,p = stats.ttest_ind(rvs1, rvs2, axis=0, equal_var = False)
+    assert_array_almost_equal([t,p],(tr,pr))
+    t,p = stats.ttest_ind(rvs1, rvs3, axis =0, equal_var = False)
+    assert_array_almost_equal([t,p], (tr_uneq_n, pr_uneq_n))
+    t,p = stats.ttest_ind(rvs1_2D.T, rvs2_2D.T, axis=0, equal_var = False)
+    assert_array_almost_equal([t,p],tpr)
+    t,p = stats.ttest_ind(rvs1_2D, rvs2_2D, axis=1, equal_var = False)
+    assert_array_almost_equal([t,p],tpr)
+
+    #test on 3 dimensions
+    rvs1_3D = np.dstack([rvs1_2D,rvs1_2D,rvs1_2D])
+    rvs2_3D = np.dstack([rvs2_2D,rvs2_2D,rvs2_2D])
+    t,p = stats.ttest_ind(rvs1_3D, rvs2_3D, axis=1, equal_var = False)
+    assert_almost_equal(np.abs(t), np.abs(tr))
+    assert_array_almost_equal(np.abs(p), pr)
+    assert_equal(t.shape, (2, 3))
+
+    t,p = stats.ttest_ind(np.rollaxis(rvs1_3D,2), np.rollaxis(rvs2_3D,2), 
+                                   axis=2, equal_var = False)
+    assert_array_almost_equal(np.abs(t), np.abs(tr))
+    assert_array_almost_equal(np.abs(p), pr)
+    assert_equal(t.shape, (3, 2))
+
+    olderr = np.seterr(all='ignore')
+    try:
+        #test zero division problem
+        t,p = stats.ttest_ind([0,0,0],[1,1,1], equal_var = False)
+        assert_equal((np.abs(t),p), (np.inf, 0))
+        assert_equal(stats.ttest_ind([0,0,0], [0,0,0], equal_var = False), (np.nan, np.nan))
+
+        #check that nan in input array result in nan output
+        anan = np.array([[1,np.nan],[-1,1]])
+        assert_equal(stats.ttest_ind(anan, np.zeros((2,2)), equal_var = False),
+                     ([0, np.nan], [1,np.nan]))
+    finally:
+        np.seterr(**olderr)
 
 def test_ttest_1samp_new():
     n1, n2, n3 = (10,15,20)
@@ -1578,7 +1683,7 @@ def test_ttest_1samp_new():
         #test zero division problem
         t,p = stats.ttest_1samp([0,0,0], 1)
         assert_equal((np.abs(t),p), (np.inf, 0))
-        assert_almost_equal(stats.ttest_1samp([0,0,0], 0), (1.0, 0.42264973081037421))
+        assert_equal(stats.ttest_1samp([0,0,0], 0), (np.nan, np.nan))
 
         #check that nan in input array result in nan output
         anan = np.array([[1,np.nan],[-1,1]])
@@ -1626,6 +1731,14 @@ def test_skewtest_too_few_samples():
     x = np.arange(7.0)
     assert_raises(ValueError, stats.skewtest, x)
 
+def test_kurtosistest_too_few_samples():
+    """Regression test for ticket #1425.
+
+    kurtosistest requires at least 5 samples; 4 should raise a ValueError.
+    """
+    x = np.arange(4.0)
+    assert_raises(ValueError, stats.kurtosistest, x)
+
 def mannwhitneyu():
     x = np.array([ 1., 1., 1., 1., 1., 1., 1., 1., 1., 1., 1., 1., 1.,
         1., 1., 1., 1., 1., 1., 1., 1., 2., 1., 1., 1., 1., 1., 1., 1.,
@@ -1960,5 +2073,65 @@ class TestFOneWay(TestCase):
         # result in F being exactly 2.0.
         assert_equal(F, 2.0)
 
+
+class TestKruskal(TestCase):
+
+    def test_simple(self):
+        """A really simple case for stats.kruskal"""
+        x = [1]
+        y = [2]
+        h, p = stats.kruskal(x, y)
+        assert_equal(h, 1.0)
+        assert_approx_equal(p, stats.chisqprob(h, 1))
+        h, p = stats.kruskal(np.array(x), np.array(y))
+        assert_equal(h, 1.0)
+        assert_approx_equal(p, stats.chisqprob(h, 1))
+
+    def test_basic(self):
+        """A basic test, with no ties."""
+        x = [1, 3, 5, 7, 9]
+        y = [2, 4, 6, 8, 10]
+        h, p = stats.kruskal(x, y)
+        assert_approx_equal(h, 3./11, significant=10)
+        assert_approx_equal(p, stats.chisqprob(3./11, 1))
+        h, p = stats.kruskal(np.array(x), np.array(y))
+        assert_approx_equal(h, 3./11, significant=10)
+        assert_approx_equal(p, stats.chisqprob(3./11, 1))
+
+    def test_simple_tie(self):
+        """A simple case with a tie."""
+        x = [1]
+        y = [1, 2]
+        h_uncorr = 1.5**2 + 2*2.25**2 - 12
+        corr = 0.75
+        expected = h_uncorr / corr   # 0.5
+        h, p = stats.kruskal(x, y)
+        # Since the expression is simple and the exact answer is 0.5, it
+        # should be safe to use assert_equal().
+        assert_equal(h, expected)
+
+    def test_another_tie(self):
+        """Another test of stats.kruskal with a tie."""
+        x = [1, 1, 1, 2]
+        y = [2, 2, 2, 2]
+        h_uncorr = (12. / 8. / 9.) * 4 * (3**2 + 6**2) - 3 * 9
+        corr = 1 - float(3**3 - 3 + 5**3 - 5) / (8**3 - 8)
+        expected = h_uncorr / corr
+        h, p = stats.kruskal(x, y)
+        assert_approx_equal(h, expected)
+
+    def test_three_groups(self):
+        """A test of stats.kruskal with three groups, with ties."""
+        x = [1, 1, 1]
+        y = [2, 2, 2]
+        z = [2, 2]
+        h_uncorr = (12. / 8. / 9.) * (3*2**2 + 3*6**2 + 2*6**2) - 3 * 9  # 5.0
+        corr = 1 - float(3**3 - 3 + 5**3 - 5) / (8**3 - 8)
+        expected = h_uncorr / corr  # 7.0
+        h, p = stats.kruskal(x, y, z)
+        assert_approx_equal(h, expected)
+        assert_approx_equal(p, stats.chisqprob(h, 2))
+
+
 if __name__ == "__main__":
     run_module_suite()
